import poser

# Global variables
EZBOOL, EZBUTTON, EZINT, EZCOLOUR, EZCOLOURP, EZFLOAT, EZCHOICE, EZSELECT, EZISELECT, EZFILE, EZLABEL, EZTEXT, EZPRESETS, EZDIR = range(14)
SKINTYPE, EYETYPE, NAILTYPE, MOUTHTYPE, TEETHTYPE, OTHERTYPE = range(6)
METHOD, SHADERTYPE = range(2)

# This class must exist - EZSkin instantiates it for building the shaders
class EZBuilder(object):

	def __init__(self, api):

		self.api = api							# A reference to an EZAPI object
		self.parameters = None 					# The method to interrogate the value of UI controls (this will be set by EZSkin)

		self.has_controls = True				# True if you supply custom controls
		self.unique_name = '02_plugin_template'	# Sorry - no naming convention yet

		# This dictionary is used to associate an EZSkin shader with a method in your classs
		# In this case, we only have one method so all are set to invoke self.MakePBR
		# The second element of the tuple determines whether or not it should be processed,
		# As selected by the user on the main EZSkin UI
		self.shaders = {
			"cornea" : (None, EYETYPE),
			"eyelash" : (None, OTHERTYPE),
			"eyetrans" : (None, EYETYPE),
			"gums" : (None, MOUTHTYPE),
			"hair" : (None, OTHERTYPE),
			"ignore" : (None, SKINTYPE),
			"innermouth" : (None, MOUTHTYPE),
			"invisible" : (None, OTHERTYPE),
			"iris" : (None, EYETYPE),
			"lacrimal" : (None, EYETYPE),
			"lips" : (None, SKINTYPE),
			"nails" : (None, NAILTYPE),
			"pupil" : (None, EYETYPE),
			"sclera" : (None, EYETYPE),
			"skin" : (None, SKINTYPE),
			"specular" : (None, SKINTYPE),
			"tear" : (None, EYETYPE),
			"teeth" : (None, TEETHTYPE),
			"tongue" : (None, MOUTHTYPE),
			'specialfx' : (self.SpecialFX, OTHERTYPE),
			}

	def _PrepareRoot(self, mat, p):
		# Used to prepare the material room for a new Superfly shader
		# In this case it will return a PoserSurface root node
		# User preferences will be taken into account (use existing root or make a new one)
		# All existing roots are managed for you

		# Get original CyclesSurface root node
		n000 = mat.roots['superfly']
		if n000:
			if n000.Type() != poser.kNodeTypeCodeCYCLESSURFACE:
				n000 = None

		# Make new root
		if p('make_root', 'settings') or n000 is None:
			n000 = mat.MakeRoot(renderer='superfly', roottype=poser.kNodeTypeCodeCYCLESSURFACE)

		return n000

	def BuilderName(self):
		# REQUIRED: returns the name of the 'shader engine' to be displayed to the user
		return 'EZSkin plugin template (SF)'

	def BuildShader(self, material, data, mat):

		# Parse the data
		shader = data[0]
		scattergroup = int(data[1])

		# Don't waste time if we don't have to do anything
		if shader == 'ignore': return 'ignored'

		# Stop if no PoserSurface is found
		if mat.Root() is None: return 'skipped - No Firefly PoserSurface found'

		# Configure imagemap nodes
		p = self.parameters
		if p('gc'): mat.GC_AllMaps()
		if p('filter'): mat.ApplyTextureFilters(p('filtering') + 1)

		# ... and build the shader
		if shader in self.Shaders():

			# Get the method which will build the shader
			method = self.shaders[shader][METHOD]

			# Check to see if we need to use a built-in shader
			if method is None and self.api.Build() >= 68:
				builder = self.api.GetBuilder(self.Renderer())
				if builder:
					return builder.BuildShader(material, data, mat)
				else:
					return 'failed'

			else:
				# Build the shader
				if method is not None:
					method(mat, p)
					mat.RemoveOrphans()
					mat.UpdatePreview()

				return 'processed'

	def ButtonExample(self, event):
		print 'Button pressed'

	def Controls(self):
		# If has_controls = True then this method must be present and return a tuple of tuples
		# (name, label, kind, minmax, default, increment=None, tooltip='', category=None)
		# See controls.txt for examples

		# Define the controls
		# (self, name, label, kind, minmax, default, increment=None, tooltip='', category=None)
		data = (
			('02_lbl01', 'Example controls', EZLABEL, (None), None, None, '', 'Main'),
			('02_bool', 'Boolean', EZBOOL, (None), 1, None, 'Boolean', 'Main'),
			('02_button', 'Button', EZBUTTON, (None), self.ButtonExample, None, 'Not applicable', 'Main'),
			('02_int', 'Integer', EZINT, (0, 10), 1, 1, 'Integer', 'Main'),
			('02_colour', 'RGB Colour', EZCOLOUR, (None), (255.0, 255.0, 255.0), None, 'Tuple', 'Main'),
			('02_colourp', 'Poser Colour', EZCOLOURP, (None), (1.0, 1.0, 1.0), None, 'Tuple', 'Main'),
			('02_float', 'Float', EZFLOAT, (0.1, 5.0), 1.0, 0.1, 'Float', 'Main'),
			('02_choice', 'Choice', EZCHOICE, ('Opt 1', 'Opt 2', 'Opt 3'), 2, None, 'Integer', 'Main'),
			('02_select', 'Select', EZSELECT, ('Opt 1', 'Opt 2', 'Opt 3'), 'Opt 2', None, 'String', 'Main'),
			('02_iselect', 'ISelect', EZISELECT, ('Opt 1', 'Opt 2', 'Opt 3'), 2, None, 'Integer', 'Main'),
			('02_file', 'File', EZFILE, (None), "", None, 'String', 'Main'),
			('02_dir', 'Folder', EZDIR, (None), "", None, 'String', 'Main'),
			('02_text', 'Text', EZTEXT, (None), 'Hello', None, 'String', 'Main'),
			)

		return data

	def Nix(self, mat, p):
		pass

	def Shaders(self):
		# Used internally by EZSkin
		return self.shaders.keys()

	def ShaderType(self, shader):
		# Used internally by EZSkin
		return self.shaders.get(shader, (None, 0))[SHADERTYPE]

	# Our single shader building method
	def SpecialFX(self, mat, p):

		# Prepare the root node
		n000 = self._PrepareRoot(mat, p)

		# For the template, we don't actually make a shader, we just print out all the ecustom control values
		ctls = self.Controls()
		for ctl in ctls:
			name = ctl[0]
			print '%s = %s' % (name, p(name))

	def Renderer(self):
		# Return the render engine this plugin is designed to work with
		# Must be 'Firefly' or 'Superfly'
		return 'Superfly'