import poser

# Global variables
EZBOOL, EZBUTTON, EZINT, EZCOLOUR, EZCOLOURP, EZFLOAT, EZCHOICE, EZSELECT, EZISELECT, EZFILE, EZLABEL, EZTEXT, EZPRESETS, EZDIR = range(14)
SKINTYPE, EYETYPE, NAILTYPE, MOUTHTYPE, TEETHTYPE, OTHERTYPE = range(6)
METHOD, SHADERTYPE = range(2)

# This class must exist - EZSkin instantiates it for building the shaders
class EZBuilder(object):

	def __init__(self, api):

		self.api = api							# A reference to an EZAPI object
		self.parameters = None 					# The method to interrogate the value of UI controls (this will be set by EZSkin)

		self.has_controls = True				# True if you supply custom controls
		self.unique_name = '01_plugin_demo'		# Sorry - no naming convention yet

		# This dictionary is used to associate an EZSkin shader with a method in your classs
		# In this case, we only have one method so all are set to invoke self.MakePBR
		# The second element of the tuple determines whether or not it should be processed,
		# As selected by the user on the main EZSkin UI
		self.shaders = {
			"cornea" : (self.MakePBR, EYETYPE),
			"eyelash" : (self.MakePBR, OTHERTYPE),
			"eyetrans" : (self.MakePBR, EYETYPE),
			"gums" : (self.MakePBR, MOUTHTYPE),
			"hair" : (self.MakePBR, OTHERTYPE),
			"ignore" : (self.Nix, SKINTYPE),
			"innermouth" : (self.MakePBR, MOUTHTYPE),
			"invisible" : (self.MakePBR, OTHERTYPE),
			"iris" : (self.MakePBR, EYETYPE),
			"lacrimal" : (self.MakePBR, EYETYPE),
			"lips" : (self.MakePBR, SKINTYPE),
			"nails" : (self.MakePBR, NAILTYPE),
			"pupil" : (self.MakePBR, EYETYPE),
			"sclera" : (self.MakePBR, EYETYPE),
			"skin" : (self.MakePBR, SKINTYPE),
			"specular" : (self.MakePBR, SKINTYPE),
			"tear" : (self.MakePBR, EYETYPE),
			"teeth" : (self.MakePBR, TEETHTYPE),
			"tongue" : (self.MakePBR, MOUTHTYPE),
			}

	def _PrepareRoot(self, mat, p):
		# Used to prepare the material room for a new Superfly shader
		# In this case it will return a PoserSurface root node
		# User preferences will be taken into account (use existing root or make a new one)
		# All existing roots are managed for you

		# Backup PS values
		mat.MakeBackup(mat.Root())

		# Make new root
		if p('make_root', 'settings'):
			n000 = mat.MakeRoot(renderer='superfly', roottype=poser.kNodeTypeCodePOSERSURFACE)
			mat.SetPSRoot(n000)
		else:
			n000 = mat.Root()

		# Copy values
		for inp in ('Bump', 'Displacement', 'Gradient_Bump', 'Gradient_Mode'):
			mat.SetPSValue(inp, mat.GetPSBackup(inp))

		return n000

	def BuilderName(self):
		# REQUIRED: returns the name of the 'shader engine' to be displayed to the user
		return 'Just the maps for exporters'

	def BuildShader(self, material, data, mat):
		# This is the method called by EZSkin for each material to be processed
		# material = a Poser Material object (included for legacy reasons)
		# data = a tuple of strings, (shadername, scattergroup)
		# mat = an EZSkin MatHelper object

		# NB: The return value of this method should be a string to be displayed in EZSkin's status bar

		# Parse the data
		shader = data[0]
		scattergroup = int(data[1])

		# Don't waste time if we don't have to do anything
		if shader == 'ignore': return 'ignored'

		# Stop if no PoserSurface is found
		if mat.Root() is None: return 'skipped - No Firefly PoserSurface found'

		# Configure imagemap nodes
		p = self.parameters
		if p('gc'): mat.GC_AllMaps()
		if p('filter'): mat.ApplyTextureFilters(p('filtering') + 1)

		# ... and build the shader
		if shader in self.Shaders():
			self.shaders[shader][METHOD](mat, p)
			mat.RemoveOrphans()
			mat.UpdatePreview()

		return 'processed'

	def Controls(self):
		# If has_controls = True then this method must be present and return a tuple of tuples
		# (name, label, kind, minmax, default, increment=None, tooltip='', category=None)
		# See controls.txt for examples

		data = (
			('01_lbl01', 'Main settings', EZLABEL, (None), None, None, '', 'Main'),
			('01_keeptrans', 'Retain transparency', EZBOOL, (None), 1, None, 'tooltip', 'Main'))
		return data

	def Nix(self, mat, p):
		pass

	def Shaders(self):
		# Used internally by EZSkin
		return self.shaders.keys()
		
	def ShaderType(self, shader):
		# Used internally by EZSkin
		return self.shaders.get(shader, (None, 0))[SHADERTYPE]

	# Our single shader building method
	def MakePBR(self, mat, p):

		# Prepare the root node
		n000 = self._PrepareRoot(mat, p)
		mat.ClearPoserSurface(keepTrans=p('01_keeptrans'), keepBump=True, use_root=n000)
		mat.SetPSValue('Diffuse_Value', 1.0)
		mat.SetPSValue('AlternateDiffuse', (1.000, 1.000, 1.000))
		mat.SetPSValue('AlternateSpecular', (1.000, 1.000, 1.000))

		# Define the channels
		channels = (
			('color', (300, 19), 'Diffuse_Color', (1,1,1), 'DIFFUSE'),
			('specular', (580, 19), 'Highlight_Value', 1.0, 'SPECULAR'),
			('bump', (300, 515), 'Bump', None, 'BUMP'),
			('displacement', (580, 515), 'Displacement', None, 'DISPLACEMENT'),
			('transparency', (300, 267), 'Transparency_Max', None, 'TRANSPARENCY'),
			('normal', (860, 515), 'Gradient_Bump', None, 'NORMAL'),
			)

		# Make the shader
		for ch in channels:
			node = mat.GetMap(ch[0])
			if node:
				mat.ConfigureNode(node, loc=ch[1], name=ch[4], showInputs=False, showPreview=True)
				mat.ConnectToPS(node, ch[2], ch[3])