import poser

# Global variables
EZBOOL, EZBUTTON, EZINT, EZCOLOUR, EZCOLOURP, EZFLOAT, EZCHOICE, EZSELECT, EZISELECT, EZFILE, EZLABEL, EZTEXT, EZPRESETS, EZDIR = range(14)
SKINTYPE, EYETYPE, NAILTYPE, MOUTHTYPE, TEETHTYPE, OTHERTYPE = range(6)
METHOD, SHADERTYPE = range(2)

# This class must exist - EZSkin instantiates it for building the shaders
class EZBuilder(object):

	def __init__(self, api):

		self.api = api							# A reference to an EZAPI object
		self.parameters = None 					# The method to interrogate the value of UI controls (this will be set by EZSkin)

		self.has_controls = False				# True if you supply custom controls
		self.unique_name = '01_plugin_demo'		# Sorry - no naming convention yet

		# This dictionary is used to associate an EZSkin shader with a method in your classs
		# In this case, we only have one method so only the cornea material is set to use the custom shader.
		# By setting all the other material to None they will be processed y the default internal shader engine.
		self.shaders = {
			"cornea" : (self.MakeCornea, EYETYPE),
			"eyelash" : (None, OTHERTYPE),
			"eyetrans" : (None, EYETYPE),
			"gums" : (None, MOUTHTYPE),
			"hair" : (None, OTHERTYPE),
			"ignore" : (None, SKINTYPE),
			"innermouth" : (None, MOUTHTYPE),
			"invisible" : (None, OTHERTYPE),
			"iris" : (None, EYETYPE),
			"lacrimal" : (None, EYETYPE),
			"lips" : (None, SKINTYPE),
			"nails" : (None, NAILTYPE),
			"pupil" : (None, EYETYPE),
			"sclera" : (None, EYETYPE),
			"skin" : (None, SKINTYPE),
			"specular" : (None, SKINTYPE),
			"tear" : (None, EYETYPE),
			"teeth" : (None, TEETHTYPE),
			"tongue" : (None, MOUTHTYPE),
			}

	def _PrepareRoot(self, mat, p):
		# Used to prepare the material room for a new Superfly shader
		# In this case it will return a PoserSurface root node
		# User preferences will be taken into account (use existing root or make a new one)
		# All existing roots are managed for you

		# Get original CyclesSurface root node
		n000 = mat.roots['superfly']
		if n000:
			if n000.Type() != poser.kNodeTypeCodeCYCLESSURFACE:
				n000 = None

		# Make new root
		if p('make_root', 'settings') or n000 is None:
			n000 = mat.MakeRoot(renderer='superfly', roottype=poser.kNodeTypeCodeCYCLESSURFACE)

		return n000

	def BuilderName(self):
		# REQUIRED: returns the name of the 'shader engine' to be displayed to the user
		return 'Tricky Glass Cornea (SF)'

	def BuildShader(self, material, data, mat):

		# Parse the data
		shader = data[0]
		scattergroup = int(data[1])

		# Don't waste time if we don't have to do anything
		if shader == 'ignore': return 'ignored'

		# Stop if no PoserSurface is found
		if mat.Root() is None: return 'skipped - No Firefly PoserSurface found'

		# Configure imagemap nodes
		p = self.parameters
		if p('gc'): mat.GC_AllMaps()
		if p('filter'): mat.ApplyTextureFilters(p('filtering') + 1)

		# ... and build the shader
		if shader in self.Shaders():

			# Get the method which will build the shader
			method = self.shaders[shader][METHOD]

			# Check to see if we need to use a built-in shader
			if method is None and self.api.Build() >= 68:
				builder = self.api.GetBuilder(self.Renderer())
				if builder:
					return builder.BuildShader(material, data, mat)
				else:
					return 'failed'

			else:
				# Build the shader
				if method is not None:
					method(mat, p)
					mat.RemoveOrphans()
					mat.UpdatePreview()

				return 'processed'

	def Nix(self, mat, p):
		pass

	def Shaders(self):
		# Used internally by EZSkin
		return self.shaders.keys()

	def ShaderType(self, shader):
		# Used internally by EZSkin
		return self.shaders.get(shader, (None, 0))[SHADERTYPE]

	# Our single shader building method
	def MakeCornea(self, mat, p):

		# Prepare the root node
		n000 = self._PrepareRoot(mat, p)

		n001 = mat.AddNode('ccl_LightPath', name='LightPath', loc=(580, 22), collapsed=False, preview=False, )
		n002 = mat.AddNode('ccl_MixClosure', name='MixClosure', loc=(300, 226), collapsed=False, preview=False, Fac=0.500, Closure1=(0.000, 0.000, 0.000), Closure2=(0.000, 0.000, 0.000), )
		n003 = mat.AddNode('ccl_GlassBsdf', name='GlassBsdf', loc=(580, 282), collapsed=False, preview=False, Color=(0.800, 0.800, 0.800), Normal=(0.000, 0.000, 0.000), Roughness=0.000, IOR=1.450, Distribution=1.000, )
		n004 = mat.AddNode('ccl_TransparentBsdf', name='TransparentBsdf', loc=(580, 415), collapsed=False, preview=False, Color=(1.000, 1.000, 1.000), )

		# Connect nodes
		mat.ConnectNodes(n002, n000, 'Surface')
		mat.ConnectNodes(n001, n002, 'Fac', output="Is Shadow Ray")
		mat.ConnectNodes(n003, n002, 'Closure1')
		mat.ConnectNodes(n004, n002, 'Closure2')

		# Define the channels
		channels = (
			('color', (300, 19), 'Diffuse_Color', (1,1,1), 'DIFFUSE'),
			('specular', (580, 19), 'Highlight_Value', 1.0, 'SPECULAR'),
			('bump', (300, 515), 'Bump', None, 'BUMP'),
			('displacement', (580, 515), 'Displacement', None, 'DISPLACEMENT'),
			('transparency', (300, 267), 'Transparency_Max', None, 'TRANSPARENCY'),
			('normal', (860, 515), 'Gradient_Bump', None, 'NORMAL'),
			)

		# Make the shader
		for ch in channels:
			node = mat.GetMap(ch[0])
			if node:
				mat.ConfigureNode(node, loc=ch[1], name=ch[4], showInputs=False, showPreview=True)
				mat.ConnectToPS(node, ch[2], ch[3])

	def Renderer(self):
		# Return the render engine this plugin is designed to work with
		# Must be 'Firefly' or 'Superfly'
		return 'Superfly'